@@ -75,7 +75,7 @@ get_activation_token(
     if (token == NULL) fail("Wayland: failed to create activation request token");
     if (_glfw.wl.activation_requests.capacity < _glfw.wl.activation_requests.sz + 1) {
         _glfw.wl.activation_requests.capacity = MAX(64u, _glfw.wl.activation_requests.capacity * 2);
-        _glfw.wl.activation_requests.array = realloc(_glfw.wl.activation_requests.array, _glfw.wl.activation_requests.capacity);
+        _glfw.wl.activation_requests.array = realloc(_glfw.wl.activation_requests.array, _glfw.wl.activation_requests.capacity * sizeof(_glfw.wl.activation_requests.array[0]));
         if (!_glfw.wl.activation_requests.array) {
             _glfw.wl.activation_requests.capacity = 0;
             fail("Wayland: Out of memory while allocation activation request");
@@ -275,18 +275,6 @@ static void setOpaqueRegion(_GLFWwindow* window, bool commit_surface)
     wl_region_destroy(region);
 }
 
-static void
-swap_buffers(_GLFWwindow *window) {
-    // this will attach the buffer to the surface,
-    // the client is responsible for clearing the buffer to an appropriate blank
-    window->swaps_disallowed = false;
-    GLFWwindow *current = glfwGetCurrentContext();
-    bool context_is_current = ((_GLFWwindow*)current)->id == window->id;
-    if (!context_is_current) glfwMakeContextCurrent((GLFWwindow*)window);
-    window->context.swapBuffers(window);
-    if (!context_is_current) glfwMakeContextCurrent(current);
-}
-
 
 static void
 resizeFramebuffer(_GLFWwindow* window) {
@@ -604,8 +592,9 @@ static void xdgSurfaceHandleConfigure(void* data,
         int width = window->wl.pending.width;
         int height = window->wl.pending.height;
         if (!window->wl.surface_configured_once) {
+            window->swaps_disallowed = false;
+            window->wl.waiting_for_swap_to_commit = true;
             window->wl.surface_configured_once = true;
-            swap_buffers(window);
         }
 
         if (new_states != window->wl.current.toplevel_states ||
@@ -628,12 +617,11 @@ static void xdgSurfaceHandleConfigure(void* data,
         window->wl.current.decoration_mode = mode;
     }
 
-    bool resized = false;
     if (window->wl.pending_state) {
         int width = window->wl.pending.width, height = window->wl.pending.height;
         set_csd_window_geometry(window, &width, &height);
-        resized = dispatchChangesAfterConfigure(window, width, height);
-        if (window->wl.decorations.serverSide) {
+        bool resized = dispatchChangesAfterConfigure(window, width, height);
+        if (window->wl.decorations.serverSide || window->monitor || window->wl.current.toplevel_states & TOPLEVEL_STATE_FULLSCREEN) {
             free_csd_surfaces(window);
         } else {
             ensure_csd_resources(window);
@@ -1650,7 +1638,7 @@ write_chunk(void *object, const char *data, size_t sz) {
     chunked_writer *cw = object;
     if (cw->cap < cw->sz + sz) {
         cw->cap = MAX(cw->cap * 2, cw->sz + 8*sz);
-        cw->buf = realloc(cw->buf, cw->cap);
+        cw->buf = realloc(cw->buf, cw->cap * sizeof(cw->buf[0]));
     }
     memcpy(cw->buf + cw->sz, data, sz);
     cw->sz += sz;
