@@ -6,15 +6,13 @@ from contextlib import suppress
 from fnmatch import fnmatch
 from functools import lru_cache
 from hashlib import md5
-from typing import (
-    TYPE_CHECKING, Dict, Iterator, List, Optional, Sequence, Set, Tuple, Union
-)
+from typing import TYPE_CHECKING, Dict, Iterator, List, Optional, Sequence, Set, Tuple, Union
 
 from kitty.guess_mime_type import guess_type
 from kitty.utils import control_codes_pat
 
 if TYPE_CHECKING:
-    from .highlight import DiffHighlight  # noqa
+    from .highlight import DiffHighlight
 
 
 path_name_map: Dict[str, str] = {}
