@@ -8,10 +8,7 @@ import sys
 from functools import lru_cache
 from gettext import gettext as _
 from itertools import repeat
-from typing import (
-    Any, Callable, Dict, Generator, Iterable, Iterator, List, Optional,
-    Pattern, Sequence, Set, Tuple, Type, cast
-)
+from typing import Any, Callable, Dict, Generator, Iterable, Iterator, List, Optional, Pattern, Sequence, Set, Tuple, Type, cast
 
 from kitty.cli import parse_args
 from kitty.cli_stub import HintsCLIOptions
@@ -20,10 +17,7 @@ from kitty.constants import website_url
 from kitty.fast_data_types import get_options, wcswidth
 from kitty.key_encoding import KeyEvent
 from kitty.typing import BossType, KittyCommonOpts
-from kitty.utils import (
-    ScreenSize, kitty_ansi_sanitizer_pat, resolve_custom_file,
-    screen_size_function
-)
+from kitty.utils import ScreenSize, kitty_ansi_sanitizer_pat, resolve_custom_file, screen_size_function
 
 from ..tui.handler import Handler, result_handler
 from ..tui.loop import Loop
