@@ -7,10 +7,7 @@ import sys
 from contextlib import suppress
 from functools import lru_cache
 from gettext import gettext as _
-from typing import (
-    Any, Dict, FrozenSet, Generator, Iterable, List, Optional, Sequence, Tuple,
-    Union
-)
+from typing import Any, Dict, FrozenSet, Generator, Iterable, List, Optional, Sequence, Tuple, Union
 
 from kitty.cli import parse_args
 from kitty.cli_stub import UnicodeCLIOptions
@@ -24,10 +21,8 @@ from kitty.utils import ScreenSize, get_editor
 from ..tui.handler import Handler, result_handler
 from ..tui.line_edit import LineEdit
 from ..tui.loop import Loop
-from ..tui.operations import (
-    clear_screen, colored, cursor, faint, set_line_wrapping, set_window_title,
-    sgr, styled
-)
+from ..tui.operations import clear_screen, colored, cursor, faint, set_line_wrapping, set_window_title, sgr, styled
+from ..tui.utils import report_unhandled_error
 
 HEX, NAME, EMOTICONS, FAVORITES = 'HEX', 'NAME', 'EMOTICONS', 'FAVORITES'
 favorites_path = os.path.join(config_dir, 'unicode-input-favorites.conf')
@@ -281,6 +276,8 @@ def is_index(w: str) -> bool:
 
 class UnicodeInput(Handler):
 
+    overlay_ready_report_needed = True
+
     def __init__(self, cached_values: Dict[str, Any], emoji_variation: str = 'none') -> None:
         self.cached_values = cached_values
         self.emoji_variation = ''
@@ -415,7 +412,7 @@ class UnicodeInput(Handler):
             if self.mode is HEX:
                 writeln(faint(_('Type {} followed by the index for the recent entries below').format(INDEX_CHAR)))
             elif self.mode is NAME:
-                writeln(faint(_('Use Tab or the arrow keys to choose a character from below')))
+                writeln(faint(_('Use Tab or arrow keys to choose a character. Type space and {} to select by index').format(INDEX_CHAR)))
             elif self.mode is FAVORITES:
                 writeln(faint(_('Press F12 to edit the list of favorites')))
             self.table_at = y
@@ -543,7 +540,7 @@ class UnicodeInput(Handler):
         self.refresh()
 
 
-help_text = 'Input a unicode character'
+help_text = 'Input a Unicode character'
 usage = ''
 OPTIONS = '''
 --emoji-variation
@@ -551,7 +548,7 @@ type=choices
 default=none
 choices=none,graphic,text
 Whether to use the textual or the graphical form for emoji. By default the
-default form specified in the unicode standard for the symbol is used.
+default form specified in the Unicode standard for the symbol is used.
 
 
 '''.format
@@ -566,8 +563,7 @@ def main(args: List[str]) -> Optional[str]:
         cli_opts, items = parse_unicode_input_args(args[1:])
     except SystemExit as e:
         if e.code != 0:
-            print(e.args[0], file=sys.stderr)
-            input(_('Press Enter to quit'))
+            report_unhandled_error(e.args[0])
         return None
 
     loop = Loop()
@@ -585,11 +581,11 @@ def main(args: List[str]) -> Optional[str]:
     return None
 
 
-@result_handler()
+@result_handler(has_ready_notification=UnicodeInput.overlay_ready_report_needed)
 def handle_result(args: List[str], current_char: str, target_window_id: int, boss: BossType) -> None:
     w = boss.window_id_map.get(target_window_id)
     if w is not None:
-        w.paste(current_char)
+        w.paste_text(current_char)
 
 
 if __name__ == '__main__':
