@@ -2,20 +2,17 @@
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 from enum import IntFlag
-from typing import Iterable, Sequence, NamedTuple
+from typing import Iterable, NamedTuple, Sequence
 
-from .fast_data_types import (
-    BORDERS_PROGRAM, add_borders_rect, compile_program, get_options,
-    init_borders_program, os_window_has_background_image
-)
+from .fast_data_types import BORDERS_PROGRAM, add_borders_rect, compile_program, get_options, init_borders_program, os_window_has_background_image
 from .typing import LayoutType
 from .utils import load_shaders
 from .window_list import WindowGroup, WindowList
 
 
 class BorderColor(IntFlag):
-    # See the border vertex shader for how these flags become actual colors
-    default_bg, active, inactive, window_bg, bell, tab_bar_bg, tab_bar_margin_color = ((1 << i) for i in range(7))
+    # These are indices into the array of colors in the border vertex shader
+    default_bg, active, inactive, window_bg, bell, tab_bar_bg, tab_bar_margin_color, tab_bar_left_edge_color, tab_bar_right_edge_color = range(9)
 
 
 class Border(NamedTuple):
