@@ -6,7 +6,7 @@ import os
 import pwd
 import sys
 from contextlib import suppress
-from typing import TYPE_CHECKING, Any, Iterable, NamedTuple, Optional, Set
+from typing import TYPE_CHECKING, Any, FrozenSet, Iterator, NamedTuple, Optional, Set
 
 from .types import run_once
 
@@ -22,13 +22,15 @@ class Version(NamedTuple):
 
 appname: str = 'kitty'
 kitty_face = '🐱'
-version: Version = Version(0, 26, 3)
+version: Version = Version(0, 27, 0)
 str_version: str = '.'.join(map(str, version))
 _plat = sys.platform.lower()
 is_macos: bool = 'darwin' in _plat
 is_freebsd: bool = 'freebsd' in _plat
 is_running_from_develop: bool = False
 RC_ENCRYPTION_PROTOCOL_VERSION = '1'
+website_base_url = 'https://sw.kovidgoyal.net/kitty/'
+default_pager_for_help = ('less', '-iRXF')
 if getattr(sys, 'frozen', False):
     extensions_dir: str = getattr(sys, 'kitty_run_data')['extensions_dir']
 
@@ -77,6 +79,10 @@ def kitty_exe() -> str:
     return os.path.join(rpath, 'kitty')
 
 
+def kitten_exe() -> str:
+    return os.path.join(os.path.dirname(kitty_exe()), 'kitten')
+
+
 def _get_config_dir() -> str:
     if 'KITTY_CONFIG_DIRECTORY' in os.environ:
         return os.path.abspath(os.path.expanduser(os.environ['KITTY_CONFIG_DIRECTORY']))
@@ -219,18 +225,31 @@ def running_in_kitty(set_val: Optional[bool] = None) -> bool:
     return bool(getattr(running_in_kitty, 'ans', False))
 
 
-def list_kitty_resources(package: str = 'kitty') -> Iterable[str]:
-    from importlib.resources import contents
-    return contents(package)
+def list_kitty_resources(package: str = 'kitty') -> Iterator[str]:
+    try:
+        if sys.version_info[:2] < (3, 10):
+            raise ImportError('importlib.resources.files() doesnt work with frozen builds on python 3.9')
+        from importlib.resources import files
+    except ImportError:
+        from importlib.resources import contents
+        return iter(contents(package))
+    else:
+        return (path.name for path in files(package).iterdir())
 
 
 def read_kitty_resource(name: str, package_name: str = 'kitty') -> bytes:
-    from importlib.resources import read_binary
-
-    return read_binary(package_name, name)
+    try:
+        if sys.version_info[:2] < (3, 10):
+            raise ImportError('importlib.resources.files() doesnt work with frozen builds on python 3.9')
+        from importlib.resources import files
+    except ImportError:
+        from importlib.resources import read_binary
+        return read_binary(package_name, name)
+    else:
+        return (files(package_name) / name).read_bytes()
 
 
-def website_url(doc_name: str = '', website: str = 'https://sw.kovidgoyal.net/kitty/') -> str:
+def website_url(doc_name: str = '', website: str = website_base_url) -> str:
     if doc_name:
         base, _, frag = doc_name.partition('#')
         base = base.rstrip('/')
@@ -277,3 +296,9 @@ def local_docs() -> str:
         if os.path.isdir(q):
             return q
     return ''
+
+
+@run_once
+def wrapped_kitten_names() -> FrozenSet[str]:
+    import kitty.fast_data_types as f
+    return frozenset(f.wrapped_kitten_names())
