@@ -18,11 +18,6 @@ def list_fonts(args: List[str]) -> None:
     list_main(args)
 
 
-def remote_control(args: List[str]) -> None:
-    from kitty.remote_control import main as rc_main
-    rc_main(args)
-
-
 def runpy(args: List[str]) -> None:
     if len(args) < 2:
         raise SystemExit('Usage: kitty +runpy "some python code"')
@@ -31,24 +26,9 @@ def runpy(args: List[str]) -> None:
 
 
 def hold(args: List[str]) -> None:
-    import subprocess
-    ret = 1
-    try:
-        ret = subprocess.Popen(args[1:]).wait()
-    except KeyboardInterrupt:
-        pass
-    except FileNotFoundError:
-        print(f'Could not find {args[1]!r} to execute', file=sys.stderr)
-    except Exception as e:
-        print(e, file=sys.stderr)
-    from kitty.utils import hold_till_enter
-    hold_till_enter()
-    raise SystemExit(ret)
-
-
-def complete(args: List[str]) -> None:
-    from kitty.complete import main as complete_main
-    complete_main(args[1:], entry_points, namespaced_entry_points)
+    from kitty.constants import kitten_exe
+    args = ['kitten', '__hold_till_enter__'] + args[1:]
+    os.execvp(kitten_exe(), args)
 
 
 def open_urls(args: List[str]) -> None:
@@ -81,6 +61,26 @@ def launch(args: List[str]) -> None:
     runpy.run_path(exe, run_name='__main__')
 
 
+def edit(args: List[str]) -> None:
+    import shutil
+
+    from .constants import is_macos
+    if is_macos:
+        # On macOS vim fails to handle SIGWINCH if it occurs early, so add a small delay.
+        import time
+        time.sleep(0.05)
+    exe = args[1]
+    if not os.path.isabs(exe):
+        exe = shutil.which(exe) or ''
+    if not exe or not os.access(exe, os.X_OK):
+        print('Cannot find an editor on your system. Set the \x1b[33meditor\x1b[39m value in kitty.conf'
+              ' to the absolute path of your editor of choice.', file=sys.stderr)
+        from kitty.utils import hold_till_enter
+        hold_till_enter()
+        raise SystemExit(1)
+    os.execv(exe, args[1:])
+
+
 def shebang(args: List[str]) -> None:
     script_path = args[1]
     cmd = args[2:]
@@ -140,18 +140,17 @@ entry_points = {
     'icat': icat,
     'list-fonts': list_fonts,
 
-    '@': remote_control,
     '+': namespaced,
 }
 namespaced_entry_points = {k: v for k, v in entry_points.items() if k[0] not in '+@'}
 namespaced_entry_points['hold'] = hold
-namespaced_entry_points['complete'] = complete
 namespaced_entry_points['runpy'] = runpy
 namespaced_entry_points['launch'] = launch
 namespaced_entry_points['open'] = open_urls
 namespaced_entry_points['kitten'] = run_kitten
 namespaced_entry_points['edit-config'] = edit_config_file
 namespaced_entry_points['shebang'] = shebang
+namespaced_entry_points['edit'] = edit
 
 
 def setup_openssl_environment(ext_dir: str) -> None:
@@ -175,9 +174,7 @@ def main() -> None:
     first_arg = '' if len(sys.argv) < 2 else sys.argv[1]
     func = entry_points.get(first_arg)
     if func is None:
-        if first_arg.startswith('@'):
-            remote_control(['@', first_arg[1:]] + sys.argv[2:])
-        elif first_arg.startswith('+'):
+        if first_arg.startswith('+'):
             namespaced(['+', first_arg[1:]] + sys.argv[2:])
         else:
             from kitty.main import main as kitty_main
