@@ -60,7 +60,7 @@ cache_size(const GraphicsManager *self) { return disk_cache_total_size(self->dis
 
 
 GraphicsManager*
-grman_alloc() {
+grman_alloc(void) {
     GraphicsManager *self = (GraphicsManager *)GraphicsManager_Type.tp_alloc(&GraphicsManager_Type, 0);
     self->images_capacity = self->capacity = 64;
     self->images = calloc(self->images_capacity, sizeof(Image));
@@ -927,8 +927,9 @@ compose(const ComposeData d, uint8_t *under_data, const uint8_t *over_data) {
 #define ROW_ITER for (unsigned y = 0; y + d.over_offset_y < d.under_height && y < d.over_height; y++) { \
         uint8_t *under_row = under_data + (y + d.over_offset_y) * d.under_px_sz * d.under_width + d.under_px_sz * d.over_offset_x; \
         const uint8_t *over_row = over_data + y * d.over_px_sz * d.over_width;
+#define END_ITER }
     if (can_copy_rows) {
-        ROW_ITER memcpy(under_row, over_row, (size_t)d.over_px_sz * min_row_sz);}
+        ROW_ITER memcpy(under_row, over_row, (size_t)d.over_px_sz * min_row_sz); END_ITER
         return;
     }
 #define PIX_ITER for (unsigned x = 0; x < min_row_sz; x++) { \
@@ -938,6 +939,7 @@ compose(const ComposeData d, uint8_t *under_data, const uint8_t *over_data) {
 #undef COPY_RGB
 #undef PIX_ITER
 #undef ROW_ITER
+#undef END_ITER
 }
 
 static CoalescedFrameData
@@ -1250,7 +1252,7 @@ handle_animation_control_command(GraphicsManager *self, bool *is_dirty, const Gr
         if (img->animation_state == ANIMATION_STOPPED) {
             img->current_loop = 0;
         } else {
-            if (old_state == ANIMATION_STOPPED) img->current_frame_shown_at = monotonic();
+            if (old_state == ANIMATION_STOPPED) { img->current_frame_shown_at = monotonic(); img->is_drawn = true; }
             self->has_images_needing_animation = true;
             global_state.check_for_active_animated_images = true;
         }
@@ -1419,12 +1421,12 @@ scroll_filter_func(ImageRef *ref, Image UNUSED *img, const void *data, CellPixel
 
 static bool
 ref_within_region(const ImageRef *ref, index_type margin_top, index_type margin_bottom) {
-    return ref->start_row >= (int32_t)margin_top && ref->start_row + ref->effective_num_rows <= margin_bottom;
+    return ref->start_row >= (int32_t)margin_top && ref->start_row + (int32_t)ref->effective_num_rows - 1 <= (int32_t)margin_bottom;
 }
 
 static bool
 ref_outside_region(const ImageRef *ref, index_type margin_top, index_type margin_bottom) {
-    return ref->start_row + ref->effective_num_rows <= margin_top || ref->start_row > (int32_t)margin_bottom;
+    return ref->start_row + (int32_t)ref->effective_num_rows <= (int32_t)margin_top || ref->start_row > (int32_t)margin_bottom;
 }
 
 static bool
@@ -1444,9 +1446,9 @@ scroll_filter_margins_func(ImageRef* ref, Image* img, const void* data, CellPixe
             ref->effective_num_rows -= clipped_rows;
             update_src_rect(ref, img);
             ref->start_row += clipped_rows;
-        } else if (ref->start_row + ref->effective_num_rows > d->margin_bottom) {
+        } else if (ref->start_row + (int32_t)ref->effective_num_rows - 1 > (int32_t)d->margin_bottom) {
             // image moved down
-            clipped_rows = ref->start_row + ref->effective_num_rows - d->margin_bottom;
+            clipped_rows = ref->start_row + ref->effective_num_rows - 1 - d->margin_bottom;
             clip_amt = cell.height * clipped_rows;
             if (ref->src_height <= clip_amt) return true;
             ref->src_height -= clip_amt;
