@@ -5,26 +5,19 @@
 import os
 import shutil
 from contextlib import suppress
-from typing import (
-    Any, Container, Dict, FrozenSet, Iterable, Iterator, List, NamedTuple,
-    Optional, Sequence, Tuple
-)
+from typing import Any, Container, Dict, FrozenSet, Iterable, Iterator, List, NamedTuple, Optional, Sequence, Tuple
 
 from .boss import Boss
 from .child import Child
 from .cli import parse_args
 from .cli_stub import LaunchCLIOptions
-from .constants import kitty_exe, shell_path
-from .fast_data_types import (
-    add_timer, get_boss, get_options, get_os_window_title,
-    patch_color_profiles, set_clipboard_string
-)
+from .clipboard import set_clipboard_string, set_primary_selection
+from .constants import kitten_exe, shell_path
+from .fast_data_types import add_timer, get_boss, get_options, get_os_window_title, patch_color_profiles
 from .options.utils import env as parse_env
 from .tabs import Tab, TabManager
 from .types import OverlayType, run_once
-from .utils import (
-    get_editor, log_error, resolve_custom_file, set_primary_selection, which
-)
+from .utils import get_editor, log_error, resolve_custom_file, which
 from .window import CwdRequest, CwdRequestType, Watchers, Window
 
 try:
@@ -95,14 +88,15 @@ opened window.
 
 
 --cwd
-completion=type:directory kwds:current,oldest,last_reported
+completion=type:directory kwds:current,oldest,last_reported,root
 The working directory for the newly launched child. Use the special value
 :code:`current` to use the working directory of the currently active window.
 The special value :code:`last_reported` uses the last working directory reported
 by the shell (needs :ref:`shell_integration` to work). The special value
 :code:`oldest` works like :code:`current` but uses the working directory of the
 oldest foreground process associated with the currently active window rather
-than the newest foreground process.
+than the newest foreground process. Finally, the special value :code:`root`
+refers to the process that was originally started when the window was created.
 
 
 --env
@@ -290,7 +284,8 @@ type=list
 Set the margin and padding for the newly created window.
 For example: :code:`margin=20` or :code:`padding-left=10` or :code:`margin-h=30`. The shorthand form sets
 all values, the :code:`*-h` and :code:`*-v` variants set horizontal and vertical values.
-Can be specified multiple times.
+Can be specified multiple times. Note that this is ignored for overlay windows as these use the settings
+from the base window.
 
 
 --watcher -w
@@ -486,6 +481,9 @@ def launch(
         elif opts.cwd == 'oldest':
             if active:
                 kw['cwd_from'] = CwdRequest(active, CwdRequestType.oldest)
+        elif opts.cwd == 'root':
+            if active:
+                kw['cwd_from'] = CwdRequest(active, CwdRequestType.root)
         else:
             kw['cwd'] = opts.cwd
     if opts.location != 'default':
@@ -546,8 +544,11 @@ def launch(
         kw['cmd'] = final_cmd
     if force_window_launch and opts.type not in non_window_launch_types:
         opts.type = 'window'
-    if opts.type in ('overlay', 'overlay-main') and active:
-        kw['overlay_for'] = active.id
+    base_for_overlay = active
+    if target_tab:
+        base_for_overlay = target_tab.active_window
+    if opts.type in ('overlay', 'overlay-main') and base_for_overlay:
+        kw['overlay_for'] = base_for_overlay.id
     if opts.type == 'background':
         cmd = kw['cmd']
         if not cmd:
@@ -563,7 +564,7 @@ def launch(
     else:
         if opts.hold:
             cmd = kw['cmd'] or [shell_path]
-            kw['cmd'] = [kitty_exe(), '+hold'] + cmd
+            kw['cmd'] = [kitten_exe(), '__hold_till_enter__'] + cmd
         if force_target_tab:
             tab = target_tab
         else:
@@ -604,39 +605,6 @@ def parse_opts_for_clone(args: List[str]) -> Tuple[LaunchCLIOptions, List[str]]:
     return default_opts, unsafe_args
 
 
-def parse_bash_env(text: str) -> Dict[str, str]:
-    # See https://www.gnu.org/software/bash/manual/html_node/Double-Quotes.html
-    ans = {}
-    pos = 0
-    escapes = r'"\$`'
-    while pos < len(text):
-        idx = text.find('="', pos)
-        if idx < 0:
-            break
-        i = text.rfind(' ', 0, idx)
-        if i < 0:
-            break
-        key = text[i+1:idx]
-        pos = idx + 2
-        buf: List[str] = []
-        a = buf.append
-        while pos < len(text):
-            ch = text[pos]
-            pos += 1
-            if ch == '\\':
-                if text[pos] in escapes:
-                    a(text[pos])
-                    pos += 1
-                    continue
-                a(ch)
-            elif ch == '"':
-                break
-            else:
-                a(ch)
-        ans[key] = ''.join(buf)
-    return ans
-
-
 def parse_null_env(text: str) -> Dict[str, str]:
     ans = {}
     for line in text.split('\0'):
@@ -670,9 +638,10 @@ class EditCmd:
         self.file_data = b''
         self.file_inode = -1, -1
         self.file_size = -1
+        self.version = 0
         self.source_window_id = self.editor_window_id = -1
         self.abort_signaled = ''
-        simple = 'file_inode', 'file_data', 'abort_signaled'
+        simple = 'file_inode', 'file_data', 'abort_signaled', 'version'
         for k, v in parse_message(msg, simple):
             if k == 'file_inode':
                 q = map(int, v.split(':'))
@@ -683,10 +652,14 @@ class EditCmd:
             elif k == 'file_data':
                 import base64
                 self.file_data = base64.standard_b64decode(v)
+            elif k == 'version':
+                self.version = int(v)
             else:
                 setattr(self, k, v)
         if self.abort_signaled:
             return
+        if self.version > 0:
+            raise ValueError(f'Unsupported version received in edit protocol: {self.version}')
         self.opts, extra_args = parse_opts_for_clone(['--type=overlay'] + self.args)
         self.file_spec = extra_args.pop()
         self.line_number = 0
@@ -775,12 +748,13 @@ class CloneCmd:
         self.shell = ''
         self.envfmt = 'default'
         self.pid = -1
+        self.bash_version = ''
         self.history = ''
         self.parse_message(msg)
         self.opts = parse_opts_for_clone(self.args)[0]
 
     def parse_message(self, msg: str) -> None:
-        simple = 'pid', 'envfmt', 'shell'
+        simple = 'pid', 'envfmt', 'shell', 'bash_version'
         for k, v in parse_message(msg, simple):
             if k in simple:
                 if k == 'pid':
@@ -790,7 +764,11 @@ class CloneCmd:
             elif k == 'a':
                 self.args.append(v)
             elif k == 'env':
-                env = parse_bash_env(v) if self.envfmt == 'bash' else parse_null_env(v)
+                if self.envfmt == 'bash':
+                    from .bash import parse_bash_env
+                    env = parse_bash_env(v, self.bash_version)
+                else:
+                    env = parse_null_env(v)
                 self.env = {k: v for k, v in env.items() if k not in {
                     'HOME', 'LOGNAME', 'USER', 'PWD',
                     # some people export these. We want the shell rc files to recreate them
@@ -851,9 +829,7 @@ def clone_and_launch(msg: str, window: Window) -> None:
     is_clone_launch = serialize_env(c.shell, env_to_serialize)
     ssh_kitten_cmdline = window.ssh_kitten_cmdline()
     if ssh_kitten_cmdline:
-        from kittens.ssh.utils import (
-            patch_cmdline, set_cwd_in_cmdline, set_env_in_cmdline
-        )
+        from kittens.ssh.utils import patch_cmdline, set_cwd_in_cmdline, set_env_in_cmdline
         cmdline = ssh_kitten_cmdline
         if c.opts.cwd:
             set_cwd_in_cmdline(c.opts.cwd, cmdline)
