@@ -3,15 +3,10 @@
 
 from functools import partial
 from itertools import repeat
-from typing import (
-    Any, Dict, Generator, Iterable, Iterator, List, NamedTuple, Optional,
-    Sequence, Tuple, Union
-)
+from typing import Any, Dict, Generator, Iterable, Iterator, List, NamedTuple, Optional, Sequence, Tuple, Union
 
 from kitty.borders import BorderColor
-from kitty.fast_data_types import (
-    Region, set_active_window, viewport_for_window
-)
+from kitty.fast_data_types import Region, set_active_window, viewport_for_window
 from kitty.options.types import Options
 from kitty.types import Edges, WindowGeometry
 from kitty.typing import TypedDict, WindowType
@@ -192,7 +187,7 @@ def safe_increment_bias(old_val: float, increment: float) -> float:
 
 def normalize_biases(biases: List[float]) -> List[float]:
     s = sum(biases)
-    if s == 1:
+    if s == 1.0:
         return biases
     return [x/s for x in biases]
 
@@ -212,7 +207,7 @@ def distribute_indexed_bias(base_bias: Sequence[float], index_bias_map: Dict[int
 
 class Layout:
 
-    name: Optional[str] = None
+    name: str = ''
     needs_window_borders = True
     must_draw_borders = False  # can be overridden to customize behavior from kittens
     layout_opts = LayoutOpts({})
