@@ -6,20 +6,17 @@ import os
 import posixpath
 import shlex
 from contextlib import suppress
-from typing import (
-    Any, Dict, Iterable, Iterator, List, NamedTuple, Optional, Tuple, cast
-)
+from typing import Any, Dict, Iterable, Iterator, List, NamedTuple, Optional, Tuple, cast
 from urllib.parse import ParseResult, unquote, urlparse
 
 from .conf.utils import KeyAction, to_cmdline_implementation
 from .constants import config_dir
+from .fast_data_types import get_options
 from .guess_mime_type import guess_type
 from .options.utils import ActionAlias, resolve_aliases_and_parse_actions
 from .types import run_once
 from .typing import MatchType
 from .utils import expandvars, get_editor, log_error, resolved_shell
-from .fast_data_types import get_options
-
 
 if not hasattr(shlex, 'join'):
     shlex.join = lambda a: ' '.join(map(shlex.quote, a))
@@ -77,7 +74,7 @@ def parse(lines: Iterable[str]) -> Iterator[OpenAction]:
     with to_cmdline_implementation.filter_env_vars(
         'URL', 'FILE_PATH', 'FILE', 'FRAGMENT', 'URL_PATH',
         EDITOR=shlex.join(get_editor()),
-        SHELL=shlex.join(resolved_shell(get_options()))
+        SHELL=resolved_shell(get_options())[0]
     ):
         for (mc, action_defns) in entries:
             actions: List[KeyAction] = []
@@ -221,6 +218,11 @@ def load_launch_actions() -> Tuple[OpenAction, ...]:
         return tuple(parse(f))
 
 
+def clear_caches() -> None:
+    load_open_actions.clear_cached()
+    load_launch_actions.clear_cached()
+
+
 @run_once
 def default_open_actions() -> Tuple[OpenAction, ...]:
     return tuple(parse('''\
@@ -232,12 +234,11 @@ action show_kitty_doc $URL_PATH
 
 @run_once
 def default_launch_actions() -> Tuple[OpenAction, ...]:
-    SHELL = resolved_shell(get_options())
-    return tuple(parse(f'''\
+    return tuple(parse('''\
 # Open script files
 protocol file
 ext sh,command,tool
-action launch --hold --type=os-window kitty +shebang $FILE_PATH {SHELL}
+action launch --hold --type=os-window kitty +shebang $FILE_PATH $SHELL
 
 # Open shell specific script files
 protocol file
