@@ -72,6 +72,11 @@ typedef struct {
     GPUCell *gpu_cells;
     bool is_active;
     index_type xstart, ynum, xnum;
+
+    struct {
+      PyObject *overlay_text;
+      const char *func_name;
+    } save;
 } OverlayLine;
 
 typedef struct {
@@ -150,6 +155,10 @@ typedef struct {
         bool is_set;
     } last_visited_prompt;
     PyObject *last_reported_cwd;
+    struct {
+        hyperlink_id_type id;
+        index_type x, y;
+    } current_hyperlink_under_mouse;
 } Screen;
 
 
@@ -261,10 +270,11 @@ void screen_push_key_encoding_flags(Screen *self, uint32_t val);
 void screen_pop_key_encoding_flags(Screen *self, uint32_t num);
 uint8_t screen_current_key_encoding_flags(Screen *self);
 void screen_report_key_encoding_flags(Screen *self);
-bool screen_detect_url(Screen *screen, unsigned int x, unsigned int y);
+int screen_detect_url(Screen *screen, unsigned int x, unsigned int y);
 int screen_cursor_at_a_shell_prompt(const Screen *);
 bool screen_fake_move_cursor_to_position(Screen *, index_type x, index_type y);
 bool screen_send_signal_for_key(Screen *, char key);
+bool get_line_edge_colors(Screen *self, color_type *left, color_type *right);
 #define DECLARE_CH_SCREEN_HANDLER(name) void screen_##name(Screen *screen);
 DECLARE_CH_SCREEN_HANDLER(bell)
 DECLARE_CH_SCREEN_HANDLER(backspace)
