@@ -1,20 +1,21 @@
 #!/usr/bin/env python3
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
+import os
 import shlex
 import sys
-from typing import (
-    TYPE_CHECKING, Generator, Iterator, List, Optional, Tuple, Union
-)
+from contextlib import suppress
+from functools import partial
+from typing import TYPE_CHECKING, Callable, Generator, Iterator, List, Mapping, Optional, Tuple, Union
 
 from .cli_stub import CLIOptions
-from .constants import kitty_exe
+from .constants import kitten_exe
 from .layout.interface import all_layouts
 from .options.types import Options
 from .options.utils import resize_window, to_layout_names, window_size
 from .os_window_size import WindowSize, WindowSizeData, WindowSizes
 from .typing import SpecialWindowInstance
-from .utils import log_error, resolved_shell, resolve_custom_file
+from .utils import expandvars, log_error, resolve_custom_file, resolved_shell
 
 if TYPE_CHECKING:
     from .launch import LaunchSpec
@@ -60,6 +61,7 @@ class Session:
         self.default_title = default_title
         self.os_window_size: Optional[WindowSizes] = None
         self.os_window_class: Optional[str] = None
+        self.focus_os_window: bool = False
 
     def add_tab(self, opts: Options, name: str = '') -> None:
         if self.tabs and not self.tabs[-1].windows:
@@ -74,11 +76,22 @@ class Session:
             raise ValueError(f'{val} is not a valid layout')
         self.tabs[-1].layout = val
 
-    def add_window(self, cmd: Union[None, str, List[str]]) -> None:
+    def add_window(self, cmd: Union[None, str, List[str]], expand: Callable[[str], str] = lambda x: x) -> None:
         from .launch import parse_launch_args
+        needs_expandvars = False
         if isinstance(cmd, str):
+            needs_expandvars = True
             cmd = shlex.split(cmd)
         spec = parse_launch_args(cmd)
+        if needs_expandvars:
+            assert isinstance(cmd, list)
+            limit = len(cmd)
+            if len(spec.args):
+                with suppress(ValueError):
+                    limit = cmd.index(spec.args[0])
+            cmd = [(expand(x) if i < limit else x) for i, x in enumerate(cmd)]
+            spec = parse_launch_args(cmd)
+
         t = self.tabs[-1]
         if t.next_title and not spec.opts.window_title:
             spec.opts.window_title = t.next_title
@@ -114,7 +127,7 @@ class Session:
         self.tabs[-1].cwd = val
 
 
-def parse_session(raw: str, opts: Options) -> Generator[Session, None, None]:
+def parse_session(raw: str, opts: Options, environ: Optional[Mapping[str, str]] = None) -> Generator[Session, None, None]:
 
     def finalize_session(ans: Session) -> Session:
         from .tabs import SpecialWindow
@@ -123,6 +136,9 @@ def parse_session(raw: str, opts: Options) -> Generator[Session, None, None]:
                 t.windows.append(WindowSpec(SpecialWindow(cmd=resolved_shell(opts))))
         return ans
 
+    if environ is None:
+        environ = os.environ
+    expand = partial(expandvars, env=environ, fallback_to_os_env=False)
     ans = Session()
     ans.add_tab(opts)
     for line in raw.splitlines():
@@ -134,6 +150,8 @@ def parse_session(raw: str, opts: Options) -> Generator[Session, None, None]:
             else:
                 cmd, rest = parts
             cmd, rest = cmd.strip(), rest.strip()
+            if cmd != 'launch':
+                rest = expand(rest)
             if cmd == 'new_tab':
                 ans.add_tab(opts, rest)
             elif cmd == 'new_os_window':
@@ -143,9 +161,11 @@ def parse_session(raw: str, opts: Options) -> Generator[Session, None, None]:
             elif cmd == 'layout':
                 ans.set_layout(rest)
             elif cmd == 'launch':
-                ans.add_window(rest)
+                ans.add_window(rest, expand)
             elif cmd == 'focus':
                 ans.focus()
+            elif cmd == 'focus_os_window':
+                ans.focus_os_window = True
             elif cmd == 'enabled_layouts':
                 ans.set_enabled_layouts(rest)
             elif cmd == 'cd':
@@ -166,9 +186,10 @@ def parse_session(raw: str, opts: Options) -> Generator[Session, None, None]:
 
 class PreReadSession(str):
 
-    def __new__(cls, val: str) -> 'PreReadSession':
+    def __new__(cls, val: str, associated_environ: Mapping[str, str]) -> 'PreReadSession':
         ans: PreReadSession = str.__new__(cls, val)
         ans.pre_read = True  # type: ignore
+        ans.associated_environ = associated_environ  # type: ignore
         return ans
 
 
@@ -178,11 +199,13 @@ def create_sessions(
     special_window: Optional['SpecialWindowInstance'] = None,
     cwd_from: Optional['CwdRequest'] = None,
     respect_cwd: bool = False,
-    default_session: Optional[str] = None
+    default_session: Optional[str] = None,
 ) -> Iterator[Session]:
     if args and args.session:
+        environ: Optional[Mapping[str, str]] = None
         if isinstance(args.session, PreReadSession):
             session_data = '' + str(args.session)
+            environ = args.session.associated_environ  # type: ignore
         else:
             if args.session == '-':
                 f = sys.stdin
@@ -190,7 +213,7 @@ def create_sessions(
                 f = open(resolve_custom_file(args.session))
             with f:
                 session_data = f.read()
-        yield from parse_session(session_data, opts)
+        yield from parse_session(session_data, opts, environ=environ)
         return
     if default_session and default_session != 'none':
         try:
@@ -208,7 +231,7 @@ def create_sessions(
     if special_window is None:
         cmd = args.args if args and args.args else resolved_shell(opts)
         if args and args.hold:
-            cmd = [kitty_exe(), '+hold'] + cmd
+            cmd = [kitten_exe(), '__hold_till_enter__'] + cmd
         from kitty.tabs import SpecialWindow
         cwd: Optional[str] = args.directory if respect_cwd and args else None
         special_window = SpecialWindow(cmd, cwd_from=cwd_from, cwd=cwd)
