@@ -52,7 +52,7 @@ typedef struct {
     BackgroundImageLayout background_image_layout;
     ImageAnchorPosition window_logo_position;
     bool background_image_linear;
-    float background_tint, window_logo_alpha;
+    float background_tint, background_tint_gaps, window_logo_alpha;
 
     bool dynamic_background_opacity;
     float inactive_text_alpha;
@@ -87,6 +87,7 @@ typedef struct {
     struct {
         float val; AdjustmentUnit unit;
     } underline_position, underline_thickness, strikethrough_position, strikethrough_thickness, cell_width, cell_height, baseline;
+    bool show_hyperlink_targets;
 } Options;
 
 typedef struct WindowLogoRenderData {
@@ -126,6 +127,14 @@ typedef struct MousePosition {
     bool in_left_half_of_cell;
 } MousePosition;
 
+typedef struct WindowBarData {
+    unsigned width, height;
+    uint8_t *buf;
+    PyObject *last_drawn_title_object_id;
+    hyperlink_id_type hyperlink_id_for_title_object;
+    bool needs_render;
+} WindowBarData;
+
 typedef struct {
     id_type id;
     bool visible, cursor_visible_at_last_render;
@@ -142,11 +151,7 @@ typedef struct {
     ClickQueue click_queues[8];
     monotonic_t last_drag_scroll_at;
     uint32_t last_special_key_pressed;
-    struct {
-        unsigned width, height;
-        uint8_t *buf;
-        PyObject *last_drawn_title_object_id;
-    } title_bar_data;
+    WindowBarData title_bar_data, url_target_bar_data;
 } Window;
 
 typedef struct {
@@ -195,6 +200,9 @@ typedef struct {
     unsigned int active_tab, num_tabs, capacity, last_active_tab, last_num_tabs, last_active_window_id;
     bool focused_at_last_render, needs_render;
     ScreenRenderData tab_bar_render_data;
+    struct {
+        color_type left, right;
+    } tab_bar_edge_color;
     bool tab_bar_data_updated;
     bool is_focused;
     monotonic_t cursor_blink_zero_time, last_mouse_activity_at;
@@ -317,6 +325,10 @@ typedef enum {
     TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY,
     TOGGLE_FULLSCREEN,
     OPEN_KITTY_WEBSITE,
+    HIDE,
+    HIDE_OTHERS,
+    MINIMIZE,
+    QUIT,
 
     NUM_COCOA_PENDING_ACTIONS
 } CocoaPendingAction;
