@@ -2,18 +2,36 @@
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
+import re
 from functools import lru_cache, partial, wraps
 from string import Formatter as StringFormatter
 from typing import (
-    Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Tuple, Union,
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
 )
 
 from .borders import Border, BorderColor
 from .config import build_ansi_color_table
 from .constants import config_dir
 from .fast_data_types import (
-    DECAWM, Color, Region, Screen, cell_size_for_window, get_boss, get_options,
-    pt_to_px, set_tab_bar_render_data, viewport_for_window,
+    DECAWM,
+    Color,
+    Region,
+    Screen,
+    cell_size_for_window,
+    get_boss,
+    get_options,
+    pt_to_px,
+    set_tab_bar_render_data,
+    update_tab_bar_edge_colors,
+    viewport_for_window,
 )
 from .rgb import alpha_blend, color_as_sgr, color_from_int, to_color
 from .types import WindowGeometry, run_once
@@ -52,6 +70,7 @@ class DrawData(NamedTuple):
     tab_activity_symbol: str
     powerline_style: PowerlineStyle
     tab_bar_edge: EdgeLiteral
+    max_tab_title_length: int
 
     def tab_fg(self, tab: TabBarData) -> int:
         if tab.is_active:
@@ -187,9 +206,24 @@ class TabAccessor:
         tab = get_boss().tab_for_id(self.tab_id)
         return (tab.get_cwd_of_active_window() if tab else '') or ''
 
+    @property
+    def active_oldest_wd(self) -> str:
+        tab = get_boss().tab_for_id(self.tab_id)
+        return (tab.get_cwd_of_active_window(oldest=True) if tab else '') or ''
+
+    @property
+    def active_exe(self) -> str:
+        tab = get_boss().tab_for_id(self.tab_id)
+        return os.path.basename((tab.get_exe_of_active_window() if tab else '') or '')
+
+    @property
+    def active_oldest_exe(self) -> str:
+        tab = get_boss().tab_for_id(self.tab_id)
+        return os.path.basename((tab.get_exe_of_active_window(oldest=True) if tab else '') or '')
+
 
 safe_builtins = {
-    'max': max, 'min': min, 'str': str, 'repr': repr, 'abs': abs, 'len': len,
+    'max': max, 'min': min, 'str': str, 'repr': repr, 'abs': abs, 'len': len, 're': re,
 }
 
 
@@ -203,6 +237,8 @@ def draw_title(draw_data: DrawData, screen: Screen, tab: TabBarData, index: int,
         'title': tab.title,
         'tab': ta,
     }
+    if draw_data.max_tab_title_length > 0:
+        max_title_length = min(max_title_length, draw_data.max_tab_title_length)
     ColorFormatter.draw_data = draw_data
     ColorFormatter.tab_data = tab
     eval_locals = {
@@ -234,7 +270,13 @@ def draw_title(draw_data: DrawData, screen: Screen, tab: TabBarData, index: int,
     except Exception as e:
         report_template_failure(template, str(e))
         title = tab.title
+    before_draw = screen.cursor.x
     draw_attributed_string(title, screen)
+    if draw_data.max_tab_title_length > 0:
+        x_limit = before_draw + draw_data.max_tab_title_length
+        if screen.cursor.x > x_limit:
+            screen.cursor.x = x_limit - 1
+            screen.draw('…')
 
 
 DrawTabFunc = Callable[[DrawData, Screen, TabBarData, int, int, int, bool, ExtraData], int]
@@ -479,7 +521,8 @@ class TabBar:
             opts.active_tab_title_template,
             opts.tab_activity_symbol,
             opts.tab_powerline_style,
-            'top' if opts.tab_bar_edge == 1 else 'bottom'
+            'top' if opts.tab_bar_edge == 1 else 'bottom',
+            opts.tab_title_max_length,
         )
         ts = opts.tab_bar_style
         if ts == 'separator':
@@ -533,6 +576,16 @@ class TabBar:
                 self.draw_data = self.draw_data._replace(inactive_fg=ifg)
         self.screen.color_profile.set_configured_colors(fg, bg)
 
+    @property
+    def current_colors(self) -> Dict[str, Color]:
+        return {
+            'active_tab_foreground': self.draw_data.active_fg,
+            'inactive_tab_foreground': self.draw_data.inactive_fg,
+            'active_tab_background': self.draw_data.active_bg,
+            'inactive_tab_background': self.draw_data.inactive_bg,
+            'tab_bar_background': self.draw_data.default_bg,
+        }
+
     def update_blank_rects(self, central: Region, tab_bar: Region, vw: int, vh: int) -> None:
         opts = get_options()
         blank_rects: List[Border] = []
@@ -549,10 +602,14 @@ class TabBar:
                 if opts.tab_bar_margin_height.inner:
                     blank_rects.append(Border(0, tab_bar.bottom + 1, vw, central.top, bg))
         g = self.window_geometry
+        left_bg = right_bg = bg
+        if opts.tab_bar_margin_color is None:
+            left_bg = BorderColor.tab_bar_left_edge_color
+            right_bg = BorderColor.tab_bar_right_edge_color
         if g.left > 0:
-            blank_rects.append(Border(0, g.top, g.left, g.bottom + 1, bg))
+            blank_rects.append(Border(0, g.top, g.left, g.bottom + 1, left_bg))
         if g.right - 1 < vw:
-            blank_rects.append(Border(g.right - 1, g.top, vw, g.bottom + 1, bg))
+            blank_rects.append(Border(g.right - 1, g.top, vw, g.bottom + 1, right_bg))
         self.blank_rects = tuple(blank_rects)
 
     def layout(self) -> None:
@@ -638,6 +695,7 @@ class TabBar:
         self.cell_ranges = cr
         s.erase_in_line(0, False)  # Ensure no long titles bleed after the last tab
         self.align()
+        update_tab_bar_edge_colors(self.os_window_id)
 
     def align_with_factor(self, factor: int = 1) -> None:
         if not self.cell_ranges:
