@@ -10,20 +10,47 @@ from contextlib import suppress
 from operator import attrgetter
 from time import monotonic
 from typing import (
-    Any, Deque, Dict, Generator, Iterable, Iterator, List,
-    NamedTuple, Optional, Sequence, Set, Tuple, Union
+    Any,
+    Deque,
+    Dict,
+    Generator,
+    Iterable,
+    Iterator,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
 )
 
 from .borders import Border, Borders
 from .child import Child
 from .cli_stub import CLIOptions
-from .constants import appname, kitty_exe
+from .constants import appname, kitten_exe
 from .fast_data_types import (
-    GLFW_MOUSE_BUTTON_LEFT, GLFW_MOUSE_BUTTON_MIDDLE, GLFW_PRESS, GLFW_RELEASE,
-    add_tab, attach_window, current_os_window, detach_window, get_boss,
-    get_click_interval, get_options, mark_tab_bar_dirty, next_window_id,
-    remove_tab, remove_window, ring_bell, set_active_tab, set_active_window,
-    swap_tabs, sync_os_window_title
+    GLFW_MOUSE_BUTTON_LEFT,
+    GLFW_MOUSE_BUTTON_MIDDLE,
+    GLFW_PRESS,
+    GLFW_RELEASE,
+    add_tab,
+    attach_window,
+    current_focused_os_window_id,
+    detach_window,
+    get_boss,
+    get_click_interval,
+    get_options,
+    last_focused_os_window_id,
+    mark_tab_bar_dirty,
+    next_window_id,
+    remove_tab,
+    remove_window,
+    ring_bell,
+    set_active_tab,
+    set_active_window,
+    swap_tabs,
+    sync_os_window_title,
 )
 from .layout.base import Layout
 from .layout.interface import create_layout_object_for, evict_cached_layouts
@@ -46,6 +73,7 @@ class TabMouseEvent(NamedTuple):
 class TabDict(TypedDict):
     id: int
     is_focused: bool
+    is_active: bool
     title: str
     layout: str
     layout_state: Dict[str, Any]
@@ -234,6 +262,10 @@ class Tab:  # {{{
         w = self.active_window
         return w.get_cwd_of_child(oldest) if w else None
 
+    def get_exe_of_active_window(self, oldest: bool = False) -> Optional[str]:
+        w = self.active_window
+        return w.get_exe_of_child(oldest) if w else None
+
     def set_title(self, title: str) -> None:
         self.name = title or ''
         self.mark_tab_bar_dirty()
@@ -354,7 +386,8 @@ class Tab:  # {{{
     def layout_action(self, action_name: str, args: Sequence[str]) -> None:
         ret = self.current_layout.layout_action(action_name, args, self.windows)
         if ret is None:
-            ring_bell()
+            if get_options().enable_audio_bell:
+                ring_bell()
             return
         self.relayout()
 
@@ -408,7 +441,7 @@ class Tab:  # {{{
                                     cmd[:0] = shlex.split(line)
                                 else:
                                     cmd[:0] = [resolved_shell(get_options())[0]]
-                                cmd[:0] = [kitty_exe(), '+hold']
+                                cmd[:0] = [kitten_exe(), '__hold_till_enter__']
         fenv: Dict[str, str] = {}
         if env:
             fenv.update(env)
@@ -490,6 +523,14 @@ class Tab:  # {{{
     def move_window_to_top_of_group(self, window: Window) -> bool:
         return self.windows.move_window_to_top_of_group(window)
 
+    def overlay_parent(self, window: Window) -> Optional[Window]:
+        prev: Optional[Window] = None
+        for x in self.windows.windows_in_group_of(window):
+            if x is window:
+                break
+            prev = x
+        return prev
+
     def remove_window(self, window: Window, destroy: bool = True) -> None:
         self.windows.remove_window(window)
         if destroy:
@@ -698,9 +739,13 @@ class Tab:  # {{{
     def move_window_backward(self) -> None:
         self.move_window(-1)
 
-    def list_windows(self, active_window: Optional[Window], self_window: Optional[Window] = None) -> Generator[WindowDict, None, None]:
+    def list_windows(self, self_window: Optional[Window] = None) -> Generator[WindowDict, None, None]:
+        active_window = self.active_window
         for w in self:
-            yield w.as_dict(is_focused=w is active_window, is_self=w is self_window)
+            yield w.as_dict(
+                is_active=w is active_window,
+                is_focused=w.os_window_id == current_focused_os_window_id() and w is active_window,
+                is_self=w is self_window)
 
     def matches_query(self, field: str, query: str, active_tab_manager: Optional['TabManager'] = None) -> bool:
         if field == 'title':
@@ -724,9 +769,10 @@ class Tab:  # {{{
             return False
         if field == 'state':
             if query == 'active':
-                return active_tab_manager is not None and self is active_tab_manager.active_tab
+                tm = self.tab_manager_ref()
+                return tm is not None and self is tm.active_tab
             if query == 'focused':
-                return active_tab_manager is not None and self is active_tab_manager.active_tab and self.os_window_id == current_os_window()
+                return active_tab_manager is not None and self is active_tab_manager.active_tab and self.os_window_id == last_focused_os_window_id()
             if query == 'needs_attention':
                 for w in self:
                     if w.needs_attention:
@@ -734,7 +780,7 @@ class Tab:  # {{{
             if query == 'parent_active':
                 return active_tab_manager is not None and self.tab_manager_ref() is active_tab_manager
             if query == 'parent_focused':
-                return active_tab_manager is not None and self.tab_manager_ref() is active_tab_manager and self.os_window_id == current_os_window()
+                return active_tab_manager is not None and self.tab_manager_ref() is active_tab_manager and self.os_window_id == last_focused_os_window_id()
             return False
         return False
 
@@ -758,7 +804,7 @@ class Tab:  # {{{
         self.windows = WindowList(self)
 
     def __repr__(self) -> str:
-        return f'Tab(title={self.effective_title}, id={hex(id(self))})'
+        return f'Tab(title={self.effective_title}, id={self.id})'
 
     def make_active(self) -> None:
         tm = self.tab_manager_ref()
@@ -934,17 +980,19 @@ class TabManager:  # {{{
     def __len__(self) -> int:
         return len(self.tabs)
 
-    def list_tabs(self, active_tab: Optional[Tab], active_window: Optional[Window], self_window: Optional[Window] = None) -> Generator[TabDict, None, None]:
+    def list_tabs(self, self_window: Optional[Window] = None) -> Generator[TabDict, None, None]:
+        active_tab = self.active_tab
         for tab in self:
             yield {
                 'id': tab.id,
-                'is_focused': tab is active_tab,
+                'is_focused': tab is active_tab and tab.os_window_id == current_focused_os_window_id(),
+                'is_active': tab is active_tab,
                 'title': tab.name or tab.title,
                 'layout': str(tab.current_layout.name),
                 'layout_state': tab.current_layout.layout_state(),
                 'layout_opts': tab.current_layout.layout_opts.serialized(),
                 'enabled_layouts': tab.enabled_layouts,
-                'windows': list(tab.list_windows(active_window, self_window)),
+                'windows': list(tab.list_windows(self_window)),
                 'active_window_history': list(tab.windows.active_window_history),
             }
 
