@@ -14,39 +14,95 @@ from gettext import gettext as _
 from itertools import chain
 from time import monotonic
 from typing import (
-    TYPE_CHECKING, Any, Callable, Deque, Dict, Iterable, List, NamedTuple,
-    Optional, Pattern, Sequence, Tuple, Union
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Deque,
+    Dict,
+    Iterable,
+    List,
+    NamedTuple,
+    Optional,
+    Pattern,
+    Sequence,
+    Tuple,
+    Union,
 )
 
 from .child import ProcessDesc
 from .cli_stub import CLIOptions
-from .clipboard import (
-    get_clipboard_string, get_primary_selection, set_clipboard_string,
-    set_primary_selection
-)
+from .clipboard import ClipboardRequestManager, set_clipboard_string
 from .config import build_ansi_color_table
 from .constants import (
-    appname, clear_handled_signals, config_dir, is_macos, wakeup_io_loop
+    appname,
+    clear_handled_signals,
+    config_dir,
+    is_macos,
+    wakeup_io_loop,
 )
 from .fast_data_types import (
-    BGIMAGE_PROGRAM, BLIT_PROGRAM, CELL_BG_PROGRAM, CELL_FG_PROGRAM,
-    CELL_PROGRAM, CELL_SPECIAL_PROGRAM, CURSOR_BEAM, CURSOR_BLOCK,
-    CURSOR_UNDERLINE, DCS, DECORATION, DECORATION_MASK, DIM, GLFW_MOD_CONTROL,
-    GRAPHICS_ALPHA_MASK_PROGRAM, GRAPHICS_PREMULT_PROGRAM, GRAPHICS_PROGRAM,
-    MARK, MARK_MASK, NO_CURSOR_SHAPE, NUM_UNDERLINE_STYLES, OSC, REVERSE,
-    SCROLL_FULL, SCROLL_LINE, SCROLL_PAGE, STRIKETHROUGH, TINT_PROGRAM, Color,
-    KeyEvent, Screen, add_timer, add_window, cell_size_for_window,
-    click_mouse_cmd_output, click_mouse_url, compile_program,
-    current_os_window, encode_key_for_tty, get_boss, get_click_interval,
-    get_options, init_cell_program, mark_os_window_dirty, mouse_selection,
-    move_cursor_to_mouse_if_in_prompt, pt_to_px, set_titlebar_color,
-    set_window_logo, set_window_padding, set_window_render_data,
-    update_ime_position_for_window, update_window_title,
-    update_window_visibility, wakeup_main_loop
+    BGIMAGE_PROGRAM,
+    BLIT_PROGRAM,
+    CELL_BG_PROGRAM,
+    CELL_FG_PROGRAM,
+    CELL_PROGRAM,
+    CELL_SPECIAL_PROGRAM,
+    CURSOR_BEAM,
+    CURSOR_BLOCK,
+    CURSOR_UNDERLINE,
+    DCS,
+    DECORATION,
+    DECORATION_MASK,
+    DIM,
+    GLFW_MOD_CONTROL,
+    GRAPHICS_ALPHA_MASK_PROGRAM,
+    GRAPHICS_PREMULT_PROGRAM,
+    GRAPHICS_PROGRAM,
+    MARK,
+    MARK_MASK,
+    NO_CURSOR_SHAPE,
+    NUM_UNDERLINE_STYLES,
+    OSC,
+    REVERSE,
+    SCROLL_FULL,
+    SCROLL_LINE,
+    SCROLL_PAGE,
+    STRIKETHROUGH,
+    TINT_PROGRAM,
+    Color,
+    KeyEvent,
+    Screen,
+    add_timer,
+    add_window,
+    cell_size_for_window,
+    click_mouse_cmd_output,
+    click_mouse_url,
+    compile_program,
+    current_focused_os_window_id,
+    encode_key_for_tty,
+    get_boss,
+    get_click_interval,
+    get_options,
+    init_cell_program,
+    last_focused_os_window_id,
+    mark_os_window_dirty,
+    mouse_selection,
+    move_cursor_to_mouse_if_in_prompt,
+    pt_to_px,
+    set_titlebar_color,
+    set_window_logo,
+    set_window_padding,
+    set_window_render_data,
+    update_ime_position_for_window,
+    update_window_title,
+    update_window_visibility,
+    wakeup_main_loop,
 )
 from .keys import keyboard_mode_name, mod_mask
 from .notify import (
-    NotificationCommand, handle_notification_cmd, sanitize_identifier_pat
+    NotificationCommand,
+    handle_notification_cmd,
+    sanitize_identifier_pat,
 )
 from .options.types import Options
 from .rgb import to_color
@@ -54,9 +110,19 @@ from .terminfo import get_capabilities
 from .types import MouseEvent, OverlayType, WindowGeometry, ac, run_once
 from .typing import BossType, ChildType, EdgeLiteral, TabType, TypedDict
 from .utils import (
-    docs_url, kitty_ansi_sanitizer_pat, load_shaders, log_error, open_cmd,
-    open_url, parse_color_set, path_from_osc7_url, resolve_custom_file,
-    resolved_shell, sanitize_title
+    docs_url,
+    kitty_ansi_sanitizer_pat,
+    load_shaders,
+    log_error,
+    open_cmd,
+    open_url,
+    parse_color_set,
+    path_from_osc7_url,
+    resolve_custom_file,
+    resolved_shell,
+    sanitize_for_bracketed_paste,
+    sanitize_title,
+    sanitize_url_for_dispay_to_user,
 )
 
 MatchPatternType = Union[Pattern[str], Tuple[Pattern[str], Optional[Pattern[str]]]]
@@ -70,6 +136,7 @@ class CwdRequestType(Enum):
     current: int = auto()
     last_reported: int = auto()
     oldest: int = auto()
+    root: int = auto()
 
 
 class CwdRequest:
@@ -93,6 +160,8 @@ class CwdRequest:
         reported_cwd = path_from_osc7_url(window.screen.last_reported_cwd) if window.screen.last_reported_cwd else ''
         if reported_cwd and not window.child_is_remote and (self.request_type is CwdRequestType.last_reported or window.at_prompt):
             return reported_cwd
+        if self.request_type is CwdRequestType.root:
+            return window.get_cwd_of_root_child() or ''
         return window.get_cwd_of_child(oldest=self.request_type is CwdRequestType.oldest) or ''
 
     def modify_argv_for_launch_with_cwd(self, argv: List[str]) -> str:
@@ -100,7 +169,7 @@ class CwdRequest:
         if not window:
             return ''
         reported_cwd = path_from_osc7_url(window.screen.last_reported_cwd) if window.screen.last_reported_cwd else ''
-        if reported_cwd:
+        if reported_cwd and (self.request_type is not CwdRequestType.root or window.root_in_foreground_processes):
             # First check if we are running ssh kitten, and trying to open the configured login shell
             if argv[0] == resolved_shell(get_options())[0]:
                 ssh_kitten_cmdline = window.ssh_kitten_cmdline()
@@ -140,6 +209,7 @@ def compile_match_query(exp: str, is_simple: bool = True) -> MatchPatternType:
 class WindowDict(TypedDict):
     id: int
     is_focused: bool
+    is_active: bool
     title: str
     pid: Optional[int]
     cwd: str
@@ -278,8 +348,6 @@ def as_text(
         h: List[str] = [pht] if pht else []
         screen.as_text_for_history_buf(h.append, as_ansi, add_wrap_markers)
         if h:
-            if not screen.linebuf.is_continued(0):
-                h[-1] += '\n'
             if as_ansi:
                 h[-1] += '\x1b[m'
         ans = ''.join(chain(h, lines))
@@ -487,6 +555,7 @@ class Window:
         else:
             self.watchers = global_watchers().copy()
         self.last_focused_at = 0.
+        self.last_resized_at = 0.
         self.started_at = monotonic()
         self.current_remote_data: List[str] = []
         self.current_mouse_event_button = 0
@@ -505,6 +574,7 @@ class Window:
         self.child_title = self.default_title
         self.title_stack: Deque[str] = deque(maxlen=10)
         self.id: int = add_window(tab.os_window_id, tab.id, self.title)
+        self.clipboard_request_manager = ClipboardRequestManager(self.id)
         self.margin = EdgeWidths()
         self.padding = EdgeWidths()
         self.kitten_result: Optional[Dict[str, Any]] = None
@@ -513,7 +583,6 @@ class Window:
         self.tab_id = tab.id
         self.os_window_id = tab.os_window_id
         self.tabref: Callable[[], Optional[TabType]] = weakref.ref(tab)
-        self.clipboard_pending: Optional[ClipboardPending] = None
         self.destroyed = False
         self.geometry: WindowGeometry = WindowGeometry(0, 0, 0, 0, 0, 0)
         self.needs_layout = True
@@ -616,10 +685,11 @@ class Window:
     def __repr__(self) -> str:
         return f'Window(title={self.title}, id={self.id})'
 
-    def as_dict(self, is_focused: bool = False, is_self: bool = False) -> WindowDict:
+    def as_dict(self, is_focused: bool = False, is_self: bool = False, is_active: bool = False) -> WindowDict:
         return dict(
             id=self.id,
             is_focused=is_focused,
+            is_active=is_active,
             title=self.title,
             pid=self.child.pid,
             cwd=self.child.current_cwd or self.child.cwd,
@@ -653,6 +723,13 @@ class Window:
             ans['overlay_type'] = self.overlay_type.value
         return ans
 
+    @property
+    def overlay_parent(self) -> Optional['Window']:
+        tab = self.tabref()
+        if tab is None:
+            return None
+        return tab.overlay_parent(self)
+
     @property
     def current_colors(self) -> Dict[str, Optional[int]]:
         return self.screen.color_profile.as_dict()
@@ -693,7 +770,7 @@ class Window:
             return False
         return False
 
-    def matches_query(self, field: str, query: str, active_tab: Optional[TabType] = None) -> bool:
+    def matches_query(self, field: str, query: str, active_tab: Optional[TabType] = None, self_window: Optional['Window'] = None) -> bool:
         if field in ('num', 'recent'):
             if active_tab is not None:
                 try:
@@ -707,15 +784,24 @@ class Window:
             return False
         if field == 'state':
             if query == 'active':
-                return active_tab is not None and self is active_tab.active_window
+                tab = self.tabref()
+                return tab is not None and tab.active_window is self
             if query == 'focused':
-                return active_tab is not None and self is active_tab.active_window and current_os_window() == self.os_window_id
+                return active_tab is not None and self is active_tab.active_window and last_focused_os_window_id() == self.os_window_id
             if query == 'needs_attention':
                 return self.needs_attention
             if query == 'parent_active':
-                return active_tab is not None and self.tabref() is active_tab
+                tab = self.tabref()
+                if tab is not None:
+                    tm = tab.tab_manager_ref()
+                    return tm is not None and tm.active_tab is tab
+                return False
             if query == 'parent_focused':
-                return active_tab is not None and self.tabref() is active_tab and current_os_window() == self.os_window_id
+                return active_tab is not None and self.tabref() is active_tab and last_focused_os_window_id() == self.os_window_id
+            if query == 'self':
+                return self is self_window
+            if query == 'overlay_parent':
+                return self_window is not None and self is self_window.overlay_parent
             return False
         pat = compile_match_query(query, field != 'env')
         return self.matches(field, pat)
@@ -746,6 +832,7 @@ class Window:
         update_ime_position = False
         if current_pty_size != self.last_reported_pty_size:
             get_boss().child_monitor.resize_pty(self.id, *current_pty_size)
+            self.last_resized_at = monotonic()
             if not self.pty_resized_once:
                 self.pty_resized_once = True
                 self.child.mark_terminal_ready()
@@ -844,11 +931,8 @@ class Window:
                 return
             if (not purl.scheme or purl.scheme == 'file'):
                 if purl.netloc:
-                    from socket import gethostname
-                    try:
-                        hostname = gethostname()
-                    except Exception:
-                        hostname = ''
+                    from .utils import get_hostname
+                    hostname = get_hostname()
                     remote_hostname = purl.netloc.partition(':')[0]
                     if remote_hostname and remote_hostname != hostname and remote_hostname != 'localhost':
                         self.handle_remote_file(purl.netloc, unquote(purl.path))
@@ -857,7 +941,7 @@ class Window:
             if opts.allow_hyperlinks & 0b10:
                 from kittens.tui.operations import styled
                 get_boss().choose(
-                    'What would you like to do with this URL:\n' + styled(unquote(url), fg='yellow'),
+                    'What would you like to do with this URL:\n' + styled(sanitize_url_for_dispay_to_user(url), fg='yellow'),
                     partial(self.hyperlink_open_confirmed, url, cwd),
                     'o:Open', 'c:Copy to clipboard', 'n;red:Nothing', default='o',
                     window=self,
@@ -921,7 +1005,7 @@ class Window:
                 tab = self.tabref()
                 if tab is not None:
                     tab.relayout_borders()
-        elif self.os_window_id == current_os_window():
+        elif self.os_window_id == current_focused_os_window_id():
             # Cancel IME composition after loses focus
             update_ime_position_for_window(self.id, False, -1)
 
@@ -941,9 +1025,12 @@ class Window:
     def has_activity_since_last_focus(self) -> bool:
         return self.screen.has_activity_since_last_focus()
 
-    def on_activity_since_last_focus(self) -> None:
-        if get_options().tab_activity_symbol:
+    def on_activity_since_last_focus(self) -> bool:
+        if get_options().tab_activity_symbol and (monotonic() - self.last_resized_at) > 0.5:
+            # Ignore activity soon after a resize as the child program is probably redrawing the screen
             get_boss().on_activity_since_last_focus(self)
+            return True
+        return False
 
     def on_bell(self) -> None:
         cb = get_options().command_on_bell
@@ -1166,86 +1253,11 @@ class Window:
     def file_transmission(self, data: str) -> None:
         self.file_transmission_control.handle_serialized_command(data)
 
-    def clipboard_control(self, data: str, is_partial: bool = False) -> None:
-        where, text = data.partition(';')[::2]
-        if is_partial:
-            if self.clipboard_pending is None:
-                self.clipboard_pending = ClipboardPending(where, text)
-            else:
-                self.clipboard_pending = self.clipboard_pending._replace(data=self.clipboard_pending[1] + text)
-                limit = get_options().clipboard_max_size
-                if limit and len(self.clipboard_pending.data) > limit * 1024 * 1024:
-                    log_error('Discarding part of too large OSC 52 paste request')
-                    self.clipboard_pending = self.clipboard_pending._replace(data='', truncated=True)
-            return
-
-        if not where:
-            if self.clipboard_pending is not None:
-                text = self.clipboard_pending.data + text
-                where = self.clipboard_pending.where
-                try:
-                    if self.clipboard_pending.truncated:
-                        return
-                finally:
-                    self.clipboard_pending = None
-            else:
-                where = 's0'
-        cc = get_options().clipboard_control
-        if text == '?':
-            response = None
-            if 's' in where or 'c' in where:
-                if 'read-clipboard-ask' in cc:
-                    return self.ask_to_read_clipboard(False)
-                response = get_clipboard_string() if 'read-clipboard' in cc else ''
-                loc = 'c'
-            elif 'p' in where:
-                if 'read-primary-ask' in cc:
-                    return self.ask_to_read_clipboard(True)
-                response = get_primary_selection() if 'read-primary' in cc else ''
-                loc = 'p'
-            response = response or ''
-            self.send_osc52(loc, response or '')
-
+    def clipboard_control(self, data: str, is_partial: Optional[bool] = False) -> None:
+        if is_partial is None:
+            self.clipboard_request_manager.parse_osc_5522(data)
         else:
-            from base64 import standard_b64decode
-            try:
-                text = standard_b64decode(text).decode('utf-8')
-            except Exception:
-                text = ''
-
-            if 's' in where or 'c' in where:
-                if 'write-clipboard' in cc:
-                    set_clipboard_string(text)
-            if 'p' in where:
-                if 'write-primary' in cc:
-                    set_primary_selection(text)
-        self.clipboard_pending = None
-
-    def send_osc52(self, loc: str, response: str) -> None:
-        from base64 import standard_b64encode
-        self.screen.send_escape_code_to_child(OSC, '52;{};{}'.format(
-            loc, standard_b64encode(response.encode('utf-8')).decode('ascii')))
-
-    def ask_to_read_clipboard(self, primary: bool = False) -> None:
-        if self.current_clipboard_read_ask is not None:
-            self.current_clipboard_read_ask = primary
-            return
-        self.current_clipboard_read_ask = primary
-        get_boss().confirm(_(
-            'A program running in this window wants to read from the system clipboard.'
-            ' Allow it do so, once?'),
-            self.handle_clipboard_confirmation, window=self,
-        )
-
-    def handle_clipboard_confirmation(self, confirmed: bool) -> None:
-        try:
-            loc = 'p' if self.current_clipboard_read_ask else 'c'
-            response = ''
-            if confirmed:
-                response = get_primary_selection() if self.current_clipboard_read_ask else get_clipboard_string()
-            self.send_osc52(loc, response)
-        finally:
-            self.current_clipboard_read_ask = None
+            self.clipboard_request_manager.parse_osc_52(data, is_partial)
 
     def manipulate_title_stack(self, pop: bool, title: str, icon: Any) -> None:
         if title:
@@ -1354,9 +1366,10 @@ class Window:
     def destroy(self) -> None:
         self.call_watchers(self.watchers.on_close, {})
         self.destroyed = True
+        self.clipboard_request_manager.close()
         del self.kitten_result_processors
         if hasattr(self, 'screen'):
-            if self.is_active and self.os_window_id == current_os_window():
+            if self.is_active and self.os_window_id == current_focused_os_window_id():
                 # Cancel IME composition when window is destroyed
                 update_ime_position_for_window(self.id, False, -1)
             # Remove cycles so that screen is de-allocated immediately
@@ -1379,10 +1392,24 @@ class Window:
     def get_cwd_of_child(self, oldest: bool = False) -> Optional[str]:
         return self.child.get_foreground_cwd(oldest) or self.child.current_cwd
 
+    def get_cwd_of_root_child(self) -> Optional[str]:
+        return self.child.current_cwd
+
+    def get_exe_of_child(self, oldest: bool = False) -> str:
+        return self.child.get_foreground_exe(oldest) or self.child.argv[0]
+
     @property
     def cwd_of_child(self) -> Optional[str]:
         return self.get_cwd_of_child()
 
+    @property
+    def root_in_foreground_processes(self) -> bool:
+        q = self.child.pid
+        for p in self.child.foreground_processes:
+            if p['pid'] == q:
+                return True
+        return False
+
     @property
     def child_is_remote(self) -> bool:
         for p in self.child.foreground_processes:
@@ -1463,11 +1490,7 @@ class Window:
             if isinstance(text, str):
                 text = text.encode('utf-8')
             if self.screen.in_bracketed_paste_mode:
-                while True:
-                    new_text = text.replace(b'\033[201~', b'').replace(b'\x9b201~', b'')
-                    if len(text) == len(new_text):
-                        break
-                    text = new_text
+                text = sanitize_for_bracketed_paste(text)
             else:
                 # Workaround for broken editors like nano that cannot handle
                 # newlines in pasted text see https://github.com/kovidgoyal/kitty/issues/994
@@ -1528,7 +1551,7 @@ class Window:
     def show_last_non_empty_command_output(self) -> None:
         self.show_cmd_output(CommandOutput.last_non_empty, 'Last non-empty command output')
 
-    @ac('cp', 'Paste the specified text into the current window')
+    @ac('cp', 'Paste the specified text into the current window. ANSI C escapes are decoded.')
     def paste(self, text: str) -> None:
         self.paste_with_actions(text)
 
@@ -1580,35 +1603,47 @@ class Window:
     def clear_selection(self) -> None:
         self.screen.clear_selection()
 
-    @ac('sc', 'Scroll up by one line')
-    def scroll_line_up(self) -> None:
+    @ac('sc', 'Scroll up by one line when in main screen')
+    def scroll_line_up(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, True)
+            return None
+        return True
 
-    @ac('sc', 'Scroll down by one line')
-    def scroll_line_down(self) -> None:
+    @ac('sc', 'Scroll down by one line when in main screen')
+    def scroll_line_down(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, False)
+            return None
+        return True
 
-    @ac('sc', 'Scroll up by one page')
-    def scroll_page_up(self) -> None:
+    @ac('sc', 'Scroll up by one page when in main screen')
+    def scroll_page_up(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, True)
+            return None
+        return True
 
-    @ac('sc', 'Scroll down by one page')
-    def scroll_page_down(self) -> None:
+    @ac('sc', 'Scroll down by one page when in main screen')
+    def scroll_page_down(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, False)
+            return None
+        return True
 
-    @ac('sc', 'Scroll to the top of the scrollback buffer')
-    def scroll_home(self) -> None:
+    @ac('sc', 'Scroll to the top of the scrollback buffer when in main screen')
+    def scroll_home(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_FULL, True)
+            return None
+        return True
 
-    @ac('sc', 'Scroll to the bottom of the scrollback buffer')
-    def scroll_end(self) -> None:
+    @ac('sc', 'Scroll to the bottom of the scrollback buffer when in main screen')
+    def scroll_end(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_FULL, False)
+            return None
+        return True
 
     @ac('sc', '''
         Scroll to the previous/next shell command prompt
@@ -1622,22 +1657,29 @@ class Window:
             map ctrl+n scroll_to_prompt 1   # jump to next
             map ctrl+o scroll_to_prompt 0   # jump to last visited
         ''')
-    def scroll_to_prompt(self, num_of_prompts: int = -1) -> None:
+    def scroll_to_prompt(self, num_of_prompts: int = -1) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll_to_prompt(num_of_prompts)
+            return None
+        return True
 
-    @ac('sc', 'Scroll prompt to the top of the screen, filling screen with empty lines')
-    def scroll_prompt_to_top(self, clear_scrollback: bool = False) -> None:
+    @ac('sc', 'Scroll prompt to the top of the screen, filling screen with empty lines, when in main screen')
+    def scroll_prompt_to_top(self, clear_scrollback: bool = False) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll_until_cursor_prompt()
             if clear_scrollback:
                 self.screen.clear_scrollback()
             elif self.screen.scrolled_by > 0:
                 self.screen.scroll(SCROLL_FULL, False)
+            return None
+        return True
 
-    @ac('sc', 'Scroll prompt to the bottom of the screen, filling in extra lines from the scrollback buffer')
-    def scroll_prompt_to_bottom(self) -> None:
-        self.screen.scroll_prompt_to_bottom()
+    @ac('sc', 'Scroll prompt to the bottom of the screen, filling in extra lines from the scrollback buffer, when in main screen')
+    def scroll_prompt_to_bottom(self) -> Optional[bool]:
+        if self.screen.is_main_linebuf():
+            self.screen.scroll_prompt_to_bottom()
+            return None
+        return True
 
     @ac('mk', 'Toggle the current marker on/off')
     def toggle_marker(self, ftype: str, spec: Union[str, Tuple[Tuple[int, str], ...]], flags: int) -> None:
